package rhu_Argue;

use strict;

# -----------------------------------------------------------------------------

my $help_string = "\n Usage:"
                . "\n"
                . "\n   rhupdate --help"
                . "\n   rhupdate --auto|manual|all [--v|--vv|--vvv]"
                . "\n "
                . "\n   Other options:"
                . "\n "
                . "\n       --listonly"
                . "\n       --checkdeps"
                . "\n ";

# -----------------------------------------------------------------------------

sub new() {

    my $class = shift @_;
    my @args  = @_;
    my $self  = { };

    bless($self, $class);

    foreach my $a (@args) {
        if ($a =~ m/^\-\-/) {

            $a =~ s/^\-\-//;

            # -- check args ok :
            unless (($a eq "auto") || ($a eq "manual") || 
                    ($a eq "checkdeps") ||
                    ($a eq "all") ||
                    ($a eq "listonly") ||
                    ($a eq "v") || ($a eq "vv") || ($a eq "vvv") ||
                    ($a eq "help")) {
                die "\n"
                  . "\n Unknown option: $a"
                  . "\n Type"
                  . "\n     rhupdate --help"
                  . "\n for some wise words."
                  . "\n\n";
              }

            if ($a eq "help") {
                print "\n" . $help_string . "\n\n";
                exit;
              }

            $a          = uc($a);
            $self->{$a} = 1;
          }
        else {
            print "\n Ignoring argument: $a";
          }
      }

    # -- deduce mode of operation :
    foreach my $k(keys %$self) {
        if (($k eq "AUTO") || ($k eq "MANUAL") || 
            ($k eq "ALL")) {
            if (defined $self->{MODE}) {
                die "\n\n Can't specify more than one mode! \n\n";
              }
            else {
                $self->{MODE} = $k;
              }
          }
      }    

    # -- ensure mode specified :
    unless (defined $self->{MODE}) {
        die "\n *** Must specify mode to be one of: " 
          . "auto, manual, all \n $help_string \n";
      }
 
    return $self;
  }

# -----------------------------------------------------------------------------
# -- return mode specified by user :

sub mode() {
    my $self = shift;

    return $self->{MODE};
  }


sub fast() {
    my $self = shift;

    if ($self->{MODE} =~ m/^FAST/) {
        return 1
      }
    else {
        return 0
      }
  }

sub all() {
    my $self = shift;

    if ($self->{MODE} eq "ALL") {
        return 1
      }
    else {
        return 0
      }
  }


sub auto() {
    my $self = shift;

    if ($self->{MODE} eq "AUTO") {
        return 1
      }
    else {
        return 0
      }
  }


sub manual() {
    my $self = shift;

    if ($self->{MODE} eq "MANUAL") {
        return 1
      }
    else {
        return 0
      }
  }


# -----------------------------------------------------------------------------

sub v() {
    my $self = shift;
    if ((defined $self->{V})   ||  (defined $self->{VV}) ||  
        (defined $self->{VVV})) {
        return 1;
      }
    else {
        return 0;
      }
  }

sub vv() {
    my $self = shift;
    if ((defined $self->{VV}) || (defined $self->{VVV})) {
        return 1;
      }
    else {
        return 0;
      }
  }


sub vvv() {
    my $self = shift;
    if (defined $self->{VVV}) {
        return 1;
      }
    else {
        return 0;
      }
  }


# -----------------------------------------------------------------------------
# -- download/listonly specified :

sub listonly() {
    my $self = shift;

    if (defined $self->{LISTONLY}) {
        return 1
      }
    else {
        return 0
      }
  }

sub checkdeps() {
    my $self = shift;

    if (defined $self->{CHECKDEPS}) {
        return 1
      }
    else {
        return 0
      }
  }

sub auto_download() {
    my $self = shift;

    if ($self->listonly() eq 0) {
        return 1
      }
    else {
        return 0;
      }
  }

sub verbosity_level() {
    my $self = shift;

    if ($self->vvv) {
        return "vvv";
      }
    if ($self->vv) {
        return "vv";
      }
    if ($self->v) {
        return "v";
      }
  }

# -----------------------------------------------------------------------------
1;
# -----------------------------------------------------------------------------
