package rhu_Download;

use strict;
use File::Copy;

use rhu_RPM;

# -- use object-oriented FTP module :
use Net::FTP;

# -----------------------------------------------------------------------------
# -- NEW :
# -----------------------------------------------------------------------------

# -- expects to be given a list of rpm _files_ to download :

sub new() {

    my $class = shift;
    my $rhu   = shift;
    my $site  = shift;
    my $files = shift;
    my $dir   = shift;

    # -- are we gonna look at dependencies/requirements of each downloaded
    #    RPM, or not --- if so, we can look at the entries in the supplied
    #    search-objects list to see we can find suitable required bits 
    #    there :
    my $recurse             = shift;
    my $search_objects_list = shift;

    my $self = { };
    bless($self, $class);

    $self->{RHU}  = $rhu;
    $self->{LIST} = $files;
    $self->{SOL}  = $search_objects_list;

    foreach my $f (@$files) {
        $self->{RHU}->log("v", 1, "simple", ["Gonna download $f->{FILE}..."]);
        &download($self, $site, $f, $dir, $recurse);
      }

    return 1;
  }


# -----------------------------------------------------------------------------
# -- PRIVATE :
# -----------------------------------------------------------------------------

sub download() {

  my $self    = shift;
  my $site    = shift;
  my $file    = shift;
  my $dir     = shift;
  my $recurse = shift;

    $self->{RHU}->log("v", 1, "simple", 
                      ["Connecting to site: $site->{NAME}..."]);
    my $ftp = Net::FTP->new($site->{NAME});

    unless (defined $ftp) { die "\n     Can't connect to ftp site."}
  
    $ftp->login($site->{LOGIN}, $site->{PASSWD});
    $self->{RHU}->log("v", 2, "simple", ["cwd: $file->{DIR}"]);
    $ftp->cwd($file->{DIR});
    $ftp->binary();

    $self->{RHU}->log("v", 2, "simple", 
                      ["Downloading to /tmp...(recurse: $recurse)"]);
    $ftp->get($file->{FILE}, "/tmp/$file->{FILE}") 
                               || die "\n     Can't get file: $file->{FILE}\n";
    $ftp->quit;

    $self->{RHU}->log("v", 1, "simple", 
                      ["Checking rpm is ok...PLACE-HOLDER only!!!"]);
    $self->{RHU}->log("v", 1, "simple", 
                      ["Moving to $dir: $file->{FILE}..."]);

    move("/tmp/$file->{FILE}", "$dir/$file->{FILE}");

    # -------------------------------------------------------------------------
    # -- BEGIN DEPENDENCIES :

    if ($recurse) {
        print "\n     Checking dependencies... ";

        my $rpm = rhu_RPM->new_file("$dir/$file->{FILE}");

        print "\n\n         (Downloaded RPM: ";
        print $rpm->{NAME}, " v", $rpm->{VERSION};
        print "\n              Requires:";

        foreach (@{$rpm->required_rpms()}) {
            print "\n                  $_->{NAME} v$_->{VERSREL}";
            print " ...checking... ";

            my $ins_rpm = rhu_RPM->new_insrpm($_->{NAME});

            # -- if installed and version/release ok :
            if (    (defined $ins_rpm) 
                 && (    ($ins_rpm->rpm_version() > $_->{VERSION})
                      || (    ($ins_rpm->rpm_version() == $_->{VERSION})
                           && ($ins_rpm->rpm_release() > $_->{RELEASE})))) {
                    print "\n                      already-installed version/release ok ($ins_rpm->{VERSION}-$ins_rpm->{RELEASE})";
	      }

            # -- elsif not previously installed, or too old :
            else {
                if (defined $ins_rpm) {
                    print "\n              *** Installed version/release " 
                        . "($ins_rpm->{VERSION}, $ins_rpm->{RELEASE}) " 
                        . "TOO OLD!!!";
                  } 
                else {
                    print "\n                  ...not previously installed...";
                  }

                # -- if suitable rpm is on our file-download-list :
                if ($self->rpm_file_on_file_download_list($_)) {
                    print "\n                  ...but it's on our " 
                        . "download-list...";
                  }
                # -- elsif we can find version/release required :
                else {
                    print "\n                  ...not on download list... "; 
                    my $found = $self->find_rpm_on_net($_);
                    if ($found) {
                       $self->download($found->{SITE}, $found->{RPM}, $dir, 1);
                      }
                    else {
                        print "\n                  *** could NOT find newer";
                      }
                  }
              } 
	  }

        print "\n         )\n";
      }

    # : END DEPENDENCIES...
    # -------------------------------------------------------------------------

    return 1;
  }


# -----------------------------------------------------------------------------
# -- TOOLKIT :
# -----------------------------------------------------------------------------


sub rpm_file_on_file_download_list() {

  my $self     = shift;
  my $req_rpm  = shift;

    my $req_rpm_name    = $req_rpm->{NAME};
    my $req_rpm_version = $req_rpm->{VERSION};
    my $req_rpm_release = $req_rpm->{RELEASE};

    $self->{RHU}->log("vvv", 4, "s", 
                      ["Is rpm ($req_rpm->{NAME}) on file-download list?"]);

    foreach my $l (@{$self->{LIST}}) {
        my $l_rpm = rhu_RPM->new_filename($l->{FILE});

	$self->{RHU}->log("vvv", 5, "s", 
                          ["...list entry: " . $l_rpm->rpm_name()]);

        if (   ($l_rpm->rpm_name() eq $req_rpm_name ) 
            && ($l_rpm->rpm_version() >= $req_rpm_version) 
            && ($l_rpm->rpm_release() >= $req_rpm_release)) {
            print "\n              On list: " . $l_rpm->rpm_name() . " "
                . $l_rpm->rpm_version() . " " . $l_rpm->rpm_release();
            return 1;
          }
      }
    return 0;
  }



sub find_rpm_on_net() {

  my $self       = shift;
  my $wanted_rpm = shift;

    my $wanted_name = $wanted_rpm->{NAME};
    my $versrel     = $wanted_rpm->{VERSREL};
    my $version     = $wanted_rpm->{VERSION};
    my $release     = $wanted_rpm->{RELEASE};

    my $found = { };

    print "\n";
    print "\n                  Attempting to find suitable rpm on Net...";

    die "\n\n NYI \n\n";

    # -- look in our search-objects lists :
    foreach my $so (@{$self->{SOL}}) {
        print "\n Looking in $so->{CLASS}";
####        if ($so->exists_name($wanted_name)) {
####            print "\n EXISTS in :  $so->{CLASS}";
####            print "\n ...need to add code to check version/release ok...";
####            print "\n ...exiting to draw attention to this fact :-) ...";
####          }
      }

    return 0;

    # -- look at ftp://updates.redhat.com, ftp://ftp.mirror.ac.uk and 
    #    ftp://ftp.redhat.com in that order :


    # -- then, perhaps, try http://rpmfind.net :

#    if () {
#        $found->{SITE} = ;
#        $found->{RPM}  = ;
#
#        return $found;
#      }
#    else {
#        return 0;
#      }
  }

# -----------------------------------------------------------------------------
1;
# -----------------------------------------------------------------------------
