package rhu_RPM;

use strict;

# -----------------------------------------------------------------------------
#
# Objects representing individual RPMs --- can handle yet-to-be installed
# RPM files and also already-installed RPMS.
#
# Also, a variant for just a RPM name.
#
# -----------------------------------------------------------------------------



# -----------------------------------------------------------------------------
# NEW (CONSTRUCTORS) :
# -----------------------------------------------------------------------------

# -- for commands only :

sub new_cmd() {
  my $class    = shift;
  
    my $self = { };

    bless($self, $class);


    return $self;
  }


# -----------------------------------------------------------------------------
# -- for an (uninstalled) rpm file (name-version.platform.rpm) :

sub new_file() {

  my $class    = shift;
  my $filename = shift;
  
    my $self = { };

    bless($self, $class);

    ($self->{NAME}, $self->{VERSION}, $self->{PLATFORM}) 
        = $self->nvp_of_rpmfile($filename);

    ($self->{REQUIREDRPMS},    $self->{REQUIREDSOS}, 
     $self->{REQUIREDRPMLIBS}, $self->{REQUIREDBINS}) 
        = $self->rpms_required_by_rpmfile($filename);

    return $self;
  }


# -----------------------------------------------------------------------------
# -- for a rpm-file-NAME only (get attributes of RPM from filename, such
#    as version and release) :

sub new_filename() {

  my $class = shift;
  my $name  = shift;
##print "\n new_filename: $name";
    my $self = { };

    bless($self, $class);

    $name =~ s/\.i\d86\.rpm$//;

    ($self->{NAME}, $self->{VERSION}, $self->{RELEASE}) = split(/\-/, $name);

    return $self;
  }


# -----------------------------------------------------------------------------
# -- for an already-installed rpm :

sub new_insrpm() {

  my $class = shift;
  my $name  = shift;
  
    my $self = { };

    bless($self, $class);

    if (`rpm -q $name 2> /dev/null` =~ m/not\ installed/) {

        $self->{NAME}    = " -- not -- installed -- ";
        $self->{VERSION} = "0";

        return undef
      }
    else {

        $self->{NAME} = $name;

        my $version = `rpm -qi $name 2> /dev/null`;
        my @version = split(/\n/, $version);

        foreach (@version) {
            if ($_ =~ m/^Version/) {
                my @fields = split(/\s+/, $_);
                $self->{VERSION} = $fields[2];
              }
          }

        my $release = `rpm -qi $name 2> /dev/null`;
        my @release = split(/\n/, $release);

        foreach (@release) {
            if ($_ =~ m/^Release/) {
                my @fields = split(/\s+/, $_);
                $self->{RELEASE} = $fields[2];
              }
          }

##        print "\n              Installed rpm: " 
##            . "$self->{NAME}, $self->{VERSION}, $self->{RELEASE}";  

        return $self;
      }
  }


# -----------------------------------------------------------------------------
# PUBLIC :
# -----------------------------------------------------------------------------

# -- returns pointer to hash of "rpms" (well packages actually, recall
#    coreutils contains textutils and fileutils plus more) each of which 
#    is a pointer to a hash containing name, vers/rel, version and release :

sub required_packages() {
  my $self = shift;

    if (defined $self->{REQUIREDRPMS}) {
        return $self->{REQUIREDRPMS};
      }
    else {
        die "\n\n rhu_RPM::required_rpms :  REQUIREDRPMS undefined \n\n";
      }
  }


sub rpm_name() {
  my $self = shift;

    if (defined $self->{NAME}) {
        return $self->{NAME};
      }
    else {
        die "\n\n rhu_RPM::required_rpms :  NAME undefined \n\n";
      }
  }


sub rpm_version() {
  my $self = shift;

    if (defined $self->{VERSION}) {
        return $self->{VERSION};
      }
    else {
        die "\n\n rhu_RPM::required_rpms :  VERSION undefined \n\n";
      }
  }


sub rpm_release() {
  my $self = shift;

    if (defined $self->{RELEASE}) {
        return $self->{RELEASE};
      }
    else {
        die "\n\n rhu_RPM::required_rpms :  RELEASE undefined \n\n";
      }
  }


# -- is the package provided by any of the already-installed RPMs ?

sub package_provided() {
    my $self    = shift;
    my $package = shift;
    my $string  = `/bin/rpm -q --whatprovides $package`;

    if ($string =~ m/no package provides/) {return 0}
    else                                   {return 1}
  }


sub package_vr() {
    my $self    = shift;
    my $package = shift;

    my $string  = `/bin/rpm -q --whatprovides $package`;
    ($string) = split(/\n/, $string);
            # ...occasionally get more than one line back, assume its
            #    the first...

    my $vr      = "";

    if ($string =~ m/no package provides/) {
        print "\n                         ...no rpm provides: $package";
        return (0, 0)
      }
    else {
        $string =~ s/\-[^\-]+$//;
        $string =~ s/\-[^\-]+$//;
        my $rpm = $string;
        print "\n                         ...rpm: $rpm, provides: $package";
        my @strs  = `/bin/rpm -q --provides $rpm`;

##########        $package =~ s/\-/\\\-/g;

        foreach (@strs) {
###print "\n p: $_";
            if ($_ =~ m/\(/) {next}
            if ($_ =~ m/$package/) {$vr = $_}
          }
##print "\n VR: $vr ($rpm, $package) ";
        unless ($vr =~ m/\=/) {return (0, 0)}

        (undef, $vr) = split(/\=/, $vr);
        if ($vr =~ m/\:/) {(undef, $vr) = split(/\:/, $vr)}
        $vr =~ s/\s*//g;
##print "\n VR: $vr ($rpm, $package) ";
        my ($v, $r) = split(/\-/, $vr);
        unless (defined $r) {$r = 0}
        return ($v, $r);
      }
  }


# -----------------------------------------------------------------------------
# PRIVATE :
# -----------------------------------------------------------------------------

# -- expects full path/filename :

sub nvp_of_rpmfile {

  my $self     = shift;
  my $filename = shift;

    my $platform = $filename;

    my $name    = `rpm -qip $filename 2> /dev/null`;
    my $version = `rpm -qip $filename 2> /dev/null`;

    my @name    = split(/\n/, $name);
    my @version = split(/\n/, $version);

    foreach (@name) {
        if ($_ =~ m/^Name/) {
            my @fields = split(/\s+/, $_);
            $name = $fields[2]
          }
      }

    foreach (@version) {
        if ($_ =~ m/^Version/) {
            my @fields = split(/\s+/, $_);
            $version = $fields[2];
          }
      }

    $platform    =~ s/\.rpm$//;
    my @platform = split(/./, $platform);
    $platform    = pop @platform;

    return ($name, $version, $platform);
  }


# -----------------------------------------------------------------------------
# -- given the filename of a RPM (name-version-release.rpm) attempts to 
#    determine on what entities it depends --- binaries, libraries (shared 
#    objects) and other RPMs :
#
#     -- each "rpm" is returned as a pointer to a hash containing name, 
#        vers/rel, version and release;

sub rpms_required_by_rpmfile {

  my $self     = shift;
  my $filename = shift;

    my $required = `rpm -qpR $filename 2> /dev/null`;
    my @required = split(/\n/, $required);

    my (@rpms, @sos, @rpmlibs, @bins);

    foreach (@required) {
        # bins...
        if ($_ =~ m/^\//) {
          }
        # ...shared objects...
        elsif (($_ =~ m/\.so\./) || ($_ =~ m/\.so\s*$/)) {
          }
        # ...RPM stuff...
        elsif ($_ =~ m/^rpmlib/) {
          }
        # ...scary stuff...
        elsif ($_ =~ m/.*\(.*\).*/) {
          }
        # ...other RPMs :
        else {
            my $rpm = { };

            ($rpm->{NAME}, undef, $rpm->{VERSREL}) = split(/\s+/, $_);

            # ...occasionally, get something odd like "kdelibs >= 6:3.1-7" :
            if ($rpm->{VERSREL} =~ m/\:/) {
                (undef, $rpm->{VERSREL}) = split(/\:/, $rpm->{VERSREL})
              }

            if (defined $rpm->{VERSREL}) {     
                ($rpm->{VERSION}, $rpm->{RELEASE}) 
                    = split(/\-/, $rpm->{VERSREL});
	      }

            unless (defined $rpm->{VERSREL}) {$rpm->{VERSREL} = "0"}
            unless (defined $rpm->{VERSION}) {$rpm->{VERSION} = "0"}
            unless (defined $rpm->{RELEASE}) {$rpm->{RELEASE} = "0"}

            push(@rpms, $rpm);
          }
      }

    return (\@rpms, \@sos, \@rpmlibs, \@bins);
  }


# -----------------------------------------------------------------------------
1;
# -----------------------------------------------------------------------------
