package rhu_RPMs_Updates;

use strict;

use vars qw(@ISA);
@ISA = ("rhu_RPMs_Template");

# -- use object-oriented FTP module :
use Net::FTP;


# -----------------------------------------------------------------------------
# -- LIST OF FTP SITES :
# -----------------------------------------------------------------------------

my @sites = ();

my $site_0 = { NAME => "updates.redhat.com",
               DIR  => "/9.0/en/os/",
               LOGIN => "anonymous",
               PASSWD => "simon.hood\@umist.ac.uk"};

my $site_1 = { NAME => "ftp.mirror.ac.uk",
               DIR  => "/sites/updates.redhat.com/9/en/os/",
               LOGIN => "anonymous",
               PASSWD => "simon.hood\@umist.ac.uk"};

my $site = $site_1;


# -----------------------------------------------------------------------------
# -- OUR ARCHITECTURE :
# -----------------------------------------------------------------------------

# -- we look in these directories and preferentially choose equivalent RPMs
#    from the architectures towards the end of the list :

my @archs = ("i386", "i486", "i586", "i686");
##@archs = ("i386", "i486", "i586", "athlon");


# -----------------------------------------------------------------------------
# -- CONSTRUCT :
# -----------------------------------------------------------------------------

# -- return an object representing the RPMS in the updates directory on 
#    the RedHat FTP site (or mirror), for, e.g., 7.2;  attributes include
#    RPM filenames, names, versions and release-numbers :

sub new() {

  my $class = shift;
  my $args  = shift;
  my $rhu   = shift;

    my $self = { };

    bless($self, $class);

    $self->{CLASS} = $class;
    $self->{RHU}   = $rhu;
    $self->{SITE}  = $site;
    $self->{ARGS}  = $args;

    # ...files keyed on file-names, files and rpm-names, and the rest (keyed on rpm-names)...
    ($self->{FILES}, $self->{FILES_2}, $self->{FILES_3}, 
     $self->{DIRS}, $self->{NAMES}, 
     $self->{ARCHS}, $self->{VERS},    $self->{RELS}) 
                                    = $self->get_list_of_RedHat_updated_rpms();

    $self->{RHU}->log("vv", 1, "simple", ["Remote updates are :"]);
    foreach (sort keys %{$self->{FILES}}) {
        $self->{RHU}->log("vv", 2, "simple", 
                          [$self->{FILES}->{$_}]);
      }   

    return $self;
  }

# -----------------------------------------------------------------------------
# -- CONSTRUCTOR SLAVES :
# -----------------------------------------------------------------------------

# -- connect anonymously to given FTP site and get list of RPM files in
#    updates directory;  from this figure out RPM names, versions and releases;
#    return name-keyed hashes of filenames, package names, versions and 
#    release numbers :

sub get_list_of_RedHat_updated_rpms() {

  my $self = shift;

    my ($files, $files_2, $files_3, 
        $dirs, $names, $archs, $versions, $releases);

    my $ftp = Net::FTP->new($site->{NAME});

    $self->{RHU}->log("v", 1, "indent_after_one",
                  ["Attempting to login to FTP site :", 
                   $site->{NAME}, $site->{LOGIN}, $site->{PASSWD}]);
    $ftp->login($site->{LOGIN}, $site->{PASSWD});

    foreach my $arch (@archs) {

        # -- attempt to change to next arch's directory, skip if missing :

        $self->{RHU}->log("v", 1, "simple",
                          ["Changing working-dir to :  $site->{DIR}$arch"]);
        $ftp->cwd($site->{DIR} . $arch);
        unless ($ftp->pwd() eq "$site->{DIR}$arch") {
            $self->{RHU}->log("v", 1, "simple",
                              [" ...skipping..."]);
            next;
          } 

        # -- get dir contents :

        $self->{RHU}->log("v", 1, "simple",
                          ["Getting dir contents..."]);
        my $dir = $ftp->dir();

        foreach (@$dir) {
            my @cols = split(/\s+/, $_);
            my $file = $cols[scalar(@cols)-1];

            my $tmp = $file;
            $tmp    =~ s/\.i\d86\.rpm$//;
            my @tmp = split(/\-/, $tmp);

            my $release = $tmp[$#tmp];
            my $version = $tmp[$#tmp - 1];

            my $name = $file;
            $name =~ s/\-$version\-$release\.i\d86\.rpm$//;

            $files->{$file} = $file;
            $dirs->{$file}  = $site->{DIR} . $arch;

            # -- "later" architecture-dirs overwrite "earlier" :

            $files_3->{$name}  = $file;
            $names->{$name}    = $name;
            $archs->{$name}    = $arch;
            $versions->{$name} = $version;
            $releases->{$name} = $release;
          }  
      }  

    $ftp->quit;

    foreach my $name (sort keys %$names) {
#        print "\n " . $name . "-" . $versions->{$name} . "-" . 
#                                    $releases->{$name} . "." . 
#                                    $archs->{$name} . ".rpm";
        my $f = $name . "-" . $versions->{$name} . "-" . 
                              $releases->{$name} . "." . 
                              $archs->{$name}    . ".rpm";
        $files_2->{$f} = $f;
      }

    return ($files, $files_2, $files_3,
            $dirs, $names, $archs, $versions, $releases);
  }


# -----------------------------------------------------------------------------
# -- PUBLIC READ METHODS :
# -----------------------------------------------------------------------------

# -- 

#sub downloadable_file_exists() {
#
#  my $self = shift;
#  my $rpm  = shift;
#    if (defined $self->{FILES}->{$rpm}) {
#        return 1;
#      }
#    else {
#        return 0;
#      }
#  }


# -----------------------------------------------------------------------------
# -- return full filename corresponding to given RPM :

sub file() {

  my $self = shift;
  my $rpm  = shift;

    if (defined $self->{FILES_3}->{$rpm}) {
        return $self->{FILES_3}->{$rpm};
      }
    else {
        die "\n\n " . (caller(0))[3] . " :  FILES_3->$rpm not defined!\n\n";
      }
  }


# -----------------------------------------------------------------------------
# --

sub exists_name() {
    my $self = shift;
    my $rpm  = shift;

    if (defined $self->{NAMES}->{$rpm}) {
        return 1;
      }
    else {
        return 0;
      }
  }

# -----------------------------------------------------------------------------
# -- return pointer to sorted list of all RPM names :

sub sorted_rpm_names() {

  my $self = shift;

    my @keys = sort keys %{$self->{NAMES}};
    return \@keys;
  }

# -----------------------------------------------------------------------------
# -- returns dir of a particular file :

sub dir() {
    my $self = shift;
    my $file = shift;

    if (defined $self->{DIRS}->{$file}) {
        return $self->{DIRS}->{$file};
      }
    else {
        die "\n\n rhu_RPMs_Updates::dir :  no DIRS for file :  $file \n\n";
      }
  }

# -----------------------------------------------------------------------------
# -- return pointer to sorted list of all RPM filenames, bit if a given
#    RPM/file comes in more than one arcitecture, return only the latest :

sub sorted_filenames_2() {

    my $self  = shift;
    my @files = ();

    foreach my $k (keys %{$self->{FILES_2}}) { 
        push(@files, $self->{FILES_2}->{$k})
      }

    @files = sort @files;

    return \@files;
  }


## -- return pointer to sorted list of all RPM filenames :
#
#sub sorted_filenames() {
#
#    my $self  = shift;
#    my @files = ();
#
#    foreach my $k (keys %{$self->{FILES}}) { 
#        push(@files, $self->{FILES}->{$k})
#      }
#
#    @files = sort @files;
#
#    return \@files;
#  }


# -----------------------------------------------------------------------------
1;
# -----------------------------------------------------------------------------
