#!/usr/bin/perl -I/home/simonh/_rpm_update

use strict;

use RHU;

use rhu_Argue;
use rhu_Download;
use rhu_Ignore;
use rhu_RPMs_Template;
use rhu_RPMs_Installed;
use rhu_RPMs_Updates;
use rhu_RPMs_Updates_Local;

# -- shut down the STDOUT buffer so can log what's going on :
select STDOUT; $| = 1;

my $installed_rpms;
    # ...leave undefined for now;

# -----------------------------------------------------------------------------
# -- GET ARGS :
# -----------------------------------------------------------------------------

my $args = rhu_Argue->new(@ARGV);
my $me   = RHU->new($args);


$me->log("v", 0, "simple", ["Mode :  " . $args->mode()]);
$me->log("v", 0, "simple", ["List only :  " . $args->listonly()]);
$me->log("v", 0, "simple", ["Auto-download :  " . $args->auto_download()]);
$me->log("v", 0, "simple", ["Verbosity :  " . $args->verbosity_level()]);


# -----------------------------------------------------------------------------
# -- GET RPMs to IGNORE, REPORT ON THESE :
# -----------------------------------------------------------------------------

my $ignores = rhu_Ignore->new($args, $me);


# -----------------------------------------------------------------------------
# -- GET LIST OF UPDATED RPMS FROM REDHAT :
# -----------------------------------------------------------------------------

$me->log("v", 0, "simple", [""]);
$me->log("v", 0, "simple", ["Getting list of updated RPMs from FTP site..."]);
my $updates = rhu_RPMs_Updates->new($args, $me);
$me->log("v", 0, "simple", 
         ["...got list of RedHat-updated RPMs from FTP site..."]);
$me->log("v", 0, "simple", [""]);


# -----------------------------------------------------------------------------
# -- GET LIST OF ALREADY-DOWNLOADED UPDATES :
# -----------------------------------------------------------------------------

$me->log("v", 0, "simple", 
        ["Checking " . $me->rootdir() 
       . " for already-downloaded update-RPMs..."]);

# -- get pointer to hash of already_downloaded rpms (in "__updates") :
my $local_updates = rhu_RPMs_Updates_Local->new($args, $me);

$me->log("v", 0, "simple", ["...got list of local update-RPMs.\n"]);


# -----------------------------------------------------------------------------
# -- LOOK FOR UPDATES NECESSARY --- MAKE A LIST (ONLY) AT THIS STAGE :
# -----------------------------------------------------------------------------

my ($u, $l, $d, $download, @downloads, @man_ignores);


# -- in "all" mode we download everything from the RedHat FTP site that
#    we've not yet got (into "__updates") and then the user should 'freshen'
#    the installation via "rpm -Fvh";

if(($args->all()) || $args->manual()) {

    # ...compare list of remote-updates to already-downloaded local-updates
    #    (and, depending on args, those already installed) and download 
    #    required...

    foreach $u (@{$updates->sorted_filenames_2()}) {
        unless (    ($local_updates->exists_file($u)) 
        ###TODO###         || (    $args->not_maintaining_updates_mirror() 
        ###TODO###              && $locals->already_installed(...))
        ) {

            # -- if manual mode, check ignore-list and ask user :
            if ($args->manual()) {
                $me->log("v", 1, "simple", [$u]);
                unless ($me->get_download("y") eq "y") { 
                    push(@man_ignores, $u);
                    next; 
                  }
              }

            my $d = { };
            $d->{FILE} = $u;
            $d->{DIR}  = $updates->dir($u);
            push(@downloads, $d);
          }
      }
  }

elsif ($args->auto()) {

    # -- get object representing locally-installed RPMs :
    $me->log("v", 0, "simple", ["Getting list of locally-installed RPMs..."]);
    $installed_rpms = rhu_RPMs_Installed->new($args, $me);
    $me->log("v", 0, "simple", ["...got list of locally-installed RPMs.\n"]);

    # ...foreach RedHat update, is there a corresponding RPM on our
    #    locally-installed-RPMs list (if not we'll do nothing)...

    $me->log("v", 0, "simple", 
             ["Checking for updates we don't already have..."]);

    foreach $u (@{$updates->sorted_rpm_names()}) {
        foreach $l (@{$installed_rpms->rpm_names()}) {
            if ($l eq $u) {

                # -- unless its on our list to be ignored or we already
                #    downloaded it to "__updates" :

                unless (    ($ignores->ignored($updates->file($u))) 
                        || ($local_updates->exists_file($updates->file($u)))) {

                    $me->log("vv", 2, "simple", 
                            ["Possible... local : " . $installed_rpms->nvr($l)]);
                    $me->log("vv", 2, "simple", 
                            ["            remote: " . $updates->nvr($u)]);

                    # -- does it look newer than our installed version?

                    if ($me->newer_versrel($updates->vers($u), 
                                           $updates->rels($u), 
                                           $installed_rpms->vers($l),
                                           $installed_rpms->rels($l))) {

                        $me->log("vv", 2, "simple", ["Newer: $u"]);

                        my $d = { };
                        $d->{FILE} = $updates->file($u);
                        $d->{DIR}  = $updates->dir($updates->file($u));
                        push(@downloads, $d);
  		      }
                    else {
                        $me->log("vv", 2, "simple", ["Same/Older: $u"]);
                      }
                  }
              }
          }
      }
    $me->log("v", 0, "simple", 
             ["...got list of updates don't already have.\n"]);
  }

# -- error :

else {
    die "\n\n What mode is this? \n\n";
  }


# -----------------------------------------------------------------------------
# -- HAVING GOT LIST OF THINGS TO DOWNLOAD, DOWNLOAD :
# -----------------------------------------------------------------------------

$me->log("v", 0, "s", 
         ["List of possible updates we are (--manual) ignoring:\n"]);
foreach (@man_ignores) {
    $me->log("v", 1, "s", ["$_"]);
  }


$me->log("v", 0, "s", ["List of updated rpms we're gonna download:\n"]);
foreach (@downloads) {
    $me->log("v", 1, "s", ["$_->{DIR}/$_->{FILE}"]);
  }


if ($args->listonly()) {
    $me->log("v", 0, "s", ["\n NOT DOWNLOADING --- LIST ONLY. \n\n"]);
  }
else {
    $me->log("v", 0, "s", ["\n Gonna download that stuff...\n"]);

    if ($d = rhu_Download->new($me,
                               $updates->{SITE}, 
                               \@downloads, 
                               $me->rootdir(), 
                               $args->checkdeps(),
                               $installed_rpms,
                               $local_updates,
                               [$updates])) {
      }
    else {
        die "Download failed...";
      }
  }  


# -----------------------------------------------------------------------------
# -- MAINTAIN UPDATES DIRECTORY :
# -----------------------------------------------------------------------------

# -- if how have two versions of something, move to "__OLD_updates" :

print "\n\n **** __OLD_updates placeholder **** \n\n";


# -----------------------------------------------------------------------------
$me->log("v", 0, "simple", ["\n Goodbye!\n\n"]);
# -----------------------------------------------------------------------------










